@@ -17,7 +17,7 @@
  *    https://dx.doi.org/10.1007/s00190-012-0578-z
  *    Addenda: http://geographiclib.sf.net/geod-addenda.html
  *
- * Copyright (c) Charles Karney (2011-2014) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2011-2015) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************
@@ -51,7 +51,8 @@
  *   GeographicLib.Geodesic.REDUCEDLENGTH
  *   GeographicLib.Geodesic.GEODESICSCALE
  *   GeographicLib.Geodesic.AREA
- *   GeographicLib.Geodesic.ALL
+ *   GeographicLib.Geodesic.ALL (all of the above)
+ *   GeographicLib.Geodesic.LONG_UNROLL
  *
  **********************************************************************
  * GeographicLib.Geodesic.WGS84.InversePath(lat1, lon1, lat2, lon2, ds12, maxk);
@@ -103,14 +104,11 @@
   g.Geodesic.CheckPosition = function(lat, lon) {
     if (!(Math.abs(lat) <= 90))
       throw new Error("latitude " + lat + " not in [-90, 90]");
-    if (!(lon >= -540 && lon < 540))
-      throw new Error("longitude " + lon + " not in [-540, 540)");
-    return m.AngNormalize(lon);
   };
 
   g.Geodesic.CheckAzimuth = function(azi) {
-    if (!(azi >= -540 && azi < 540))
-      throw new Error("longitude " + azi + " not in [-540, 540)");
+    if (!(isFinite(azi)))
+      throw new Error("azimuth " + azi + " not a finite number");
     return m.AngNormalize(azi);
   };
 
@@ -121,27 +119,17 @@
 
   g.Geodesic.prototype.Inverse = function(lat1, lon1, lat2, lon2, outmask) {
     if (!outmask) outmask = g.DISTANCE | g.AZIMUTH;
-    lon1 = g.Geodesic.CheckPosition(lat1, lon1);
-    lon2 = g.Geodesic.CheckPosition(lat2, lon2);
-
-    var result = this.GenInverse(lat1, lon1, lat2, lon2, outmask);
-    result.lat1 = lat1; result.lon1 = lon1;
-    result.lat2 = lat2; result.lon2 = lon2;
-
-    return result;
+    g.Geodesic.CheckPosition(lat1, lon1);
+    g.Geodesic.CheckPosition(lat2, lon2);
+    return this.GenInverse(lat1, lon1, lat2, lon2, outmask);
   };
 
   g.Geodesic.prototype.Direct = function(lat1, lon1, azi1, s12, outmask) {
     if (!outmask) outmask = g.LATITUDE | g.LONGITUDE | g.AZIMUTH;
-    lon1 = g.Geodesic.CheckPosition(lat1, lon1);
-    azi1 = g.Geodesic.CheckAzimuth(azi1);
+    g.Geodesic.CheckPosition(lat1, lon1);
+    g.Geodesic.CheckAzimuth(azi1);
     g.Geodesic.CheckDistance(s12);
-
-    var result = this.GenDirect(lat1, lon1, azi1, false, s12, outmask);
-    result.lat1 = lat1; result.lon1 = lon1;
-    result.azi1 = azi1; result.s12 = s12;
-
-    return result;
+    return this.GenDirect(lat1, lon1, azi1, false, s12, outmask);
   };
 
   g.Geodesic.prototype.InversePath =
@@ -197,10 +185,6 @@
   g.Geodesic.prototype.Circle = function(lat1, lon1, azi1, s12, k) {
     if (!(Math.abs(lat1) <= 90))
       throw new Error("lat1 must be in [-90, 90]");
-    if (!(lon1 >= -540 && lon1 < 540))
-      throw new Error("lon1 must be in [-540, 540)");
-    if (!(azi1 >= -540 && azi1 < 540))
-      throw new Error("azi1 must be in [-540, 540)");
     if (!(isFinite(s12)))
       throw new Error("s12 must be a finite number");
     lon1 = m.AngNormalize(lon1);
@@ -221,8 +205,6 @@
   g.Geodesic.prototype.Envelope = function(lat1, lon1, k, ord) {
     if (!(Math.abs(lat1) <= 90))
       throw new Error("lat1 must be in [-90, 90]");
-    if (!(lon1 >= -540 && lon1 < 540))
-      throw new Error("lon1 must be in [-540, 540)");
     lon1 = m.AngNormalize(lon1);
     if (!k || k < 4) k = 24;
     if (!ord) ord = 1;
