@@ -96,7 +96,7 @@ namespace NETGeographicLib
           CAP_ALL  = 0x1FU,
           CAP_MASK = CAP_ALL,
           OUT_ALL  = 0x7F80U,
-          OUT_MASK = 0xFF80U,       // Includes LONG_NOWRAP
+          OUT_MASK = 0xFF80U,       // Includes LONG_UNROLL
         };
         // pointer to the unmanaged GeographicLib::GeodesicExact.
         const GeographicLib::GeodesicExact* m_pGeodesicExact;
@@ -164,12 +164,14 @@ namespace NETGeographicLib
            **********************************************************************/
           AREA          = 1U<<14 | unsigned(captype::CAP_C4),
           /**
-           * Do not wrap the \e lon2 in the direct calculation.
+           * Unroll \e lon2 in the direct calculation.  (This flag used to be
+           * called LONG_NOWRAP.)
            * @hideinitializer
            **********************************************************************/
-          LONG_NOWRAP   = 1U<<15,
+          LONG_UNROLL   = 1U<<15,
+          LONG_NOWRAP   = LONG_UNROLL,
           /**
-           * All capabilities, calculate everything.  (LONG_NOWRAP is not
+           * All capabilities, calculate everything.  (LONG_UNROLL is not
            * included in this mask.)
            * @hideinitializer
            **********************************************************************/
@@ -226,10 +228,9 @@ namespace NETGeographicLib
          * @param[out] S12 area under the geodesic (meters<sup>2</sup>).
          * @return \e a12 arc length of between point 1 and point 2 (degrees).
          *
-         * \e lat1 should be in the range [&minus;90&deg;, 90&deg;]; \e lon1 and \e
-         * azi1 should be in the range [&minus;540&deg;, 540&deg;).  The values of
-         * \e lon2 and \e azi2 returned are in the range [&minus;180&deg;,
-         * 180&deg;).
+         * \e lat1 should be in the range [&minus;90&deg;, 90&deg;];.  The
+         * values of \e lon2 and \e azi2 returned are in the range
+         * [&minus;180&deg;, 180&deg;).
          *
          * If either point is at a pole, the azimuth is defined by keeping the
          * longitude fixed, writing \e lat = &plusmn;(90&deg; &minus; &epsilon;),
@@ -320,10 +321,9 @@ namespace NETGeographicLib
          *   (dimensionless).
          * @param[out] S12 area under the geodesic (meters<sup>2</sup>).
          *
-         * \e lat1 should be in the range [&minus;90&deg;, 90&deg;]; \e lon1 and \e
-         * azi1 should be in the range [&minus;540&deg;, 540&deg;).  The values of
-         * \e lon2 and \e azi2 returned are in the range [&minus;180&deg;,
-         * 180&deg;).
+         * \e lat1 should be in the range [&minus;90&deg;, 90&deg;].  The
+         * values of \e lon2 and \e azi2 returned are in the range
+         * [&minus;180&deg;, 180&deg;).
          *
          * If either point is at a pole, the azimuth is defined by keeping the
          * longitude fixed, writing \e lat = &plusmn;(90&deg; &minus; &epsilon;),
@@ -444,8 +444,8 @@ namespace NETGeographicLib
          *   M12 and \e M21;
          * - \e outmask |= GeodesicExact::AREA for the area \e S12;
          * - \e outmask |= GeodesicExact::ALL for all of the above;
-         * - \e outmask |= GeodesicExact::LONG_NOWRAP stops the returned value of
-         *   \e lon2 being wrapped into the range [&minus;180&deg;, 180&deg;).
+         * - \e outmask |= GeodesicExact::LONG_UNROLL to unroll \e lon2 instead of
+         *   wrapping it into the range [&minus;180&deg;, 180&deg;).
          * .
          * The function value \e a12 is always computed and returned and this
          * equals \e s12_a12 is \e arcmode is true.  If \e outmask includes
@@ -453,11 +453,9 @@ namespace NETGeographicLib
          * s12_a12.  It is not necessary to include GeodesicExact::DISTANCE_IN in
          * \e outmask; this is automatically included is \e arcmode is false.
          *
-         * With the LONG_NOWRAP bit set, the quantity \e lon2 &minus; \e lon1
-         * indicates how many times the geodesic wrapped around the ellipsoid.
-         * Because \e lon2 might be outside the normal allowed range for
-         * longitudes, [&minus;540&deg;, 540&deg;), be sure to normalize it with
-         * Math::AngNormalize2 before using it in other GeographicLib calls.
+         * With the LONG_UNROLL bit set, the quantity \e lon2 &minus; \e lon1
+         * indicates how many times and in what sense the geodesic encircles
+         * the ellipsoid.
          **********************************************************************/
         double GenDirect(double lat1, double lon1, double azi1,
                         bool arcmode, double s12_a12, GeodesicExact::mask outmask,
@@ -492,10 +490,9 @@ namespace NETGeographicLib
          * @param[out] S12 area under the geodesic (meters<sup>2</sup>).
          * @return \e a12 arc length of between point 1 and point 2 (degrees).
          *
-         * \e lat1 and \e lat2 should be in the range [&minus;90&deg;, 90&deg;]; \e
-         * lon1 and \e lon2 should be in the range [&minus;540&deg;, 540&deg;).
-         * The values of \e azi1 and \e azi2 returned are in the range
-         * [&minus;180&deg;, 180&deg;).
+         * \e lat1 and \e lat2 should be in the range [&minus;90&deg;,
+         * 90&deg;].  The values of \e azi1 and \e azi2 returned are in the
+         * range [&minus;180&deg;, 180&deg;).
          *
          * If either point is at a pole, the azimuth is defined by keeping the
          * longitude fixed, writing \e lat = &plusmn;(90&deg; &minus; &epsilon;),
@@ -629,8 +626,7 @@ namespace NETGeographicLib
          *   GeodesicLineExact::Position.
          * @return a GeodesicLineExact object.
          *
-         * \e lat1 should be in the range [&minus;90&deg;, 90&deg;]; \e lon1 and \e
-         * azi1 should be in the range [&minus;540&deg;, 540&deg;).
+         * \e lat1 should be in the range [&minus;90&deg;, 90&deg;].
          *
          * The GeodesicExact::mask values are
          * - \e caps |= NETGeographicLib::Mask::LATITUDE for the latitude \e lat2; this is
