@@ -107,12 +107,14 @@ namespace NETGeographicLib {
        **********************************************************************/
       AREA          = 1U<<14,
       /**
-       * Do not wrap the \e lon2 in the direct calculation.
+       * Unroll \e lon2 in the direct calculation.  (This flag used to be
+       * called LONG_NOWRAP.)
        * @hideinitializer
        **********************************************************************/
-      LONG_NOWRAP   = 1U<<15,
+      LONG_UNROLL   = 1U<<15,
+      LONG_NOWRAP   = LONG_UNROLL,
       /**
-       * Calculate everything.  (LONG_NOWRAP is not included in this mask.)
+       * Calculate everything.  (LONG_UNROLL is not included in this mask.)
        * @hideinitializer
        **********************************************************************/
       ALL           = 0x7F80U,
@@ -152,8 +154,7 @@ namespace NETGeographicLib {
      * @param[out] lon2 longitude of point 2 (degrees).
      * @param[out] S12 area under the rhumb line (meters<sup>2</sup>).
      *
-     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;]; \e lon1 and \e
-     * azi12 should be in the range [&minus;540&deg;, 540&deg;).  The value of
+     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;].  The value of
      * \e lon2 returned is in the range [&minus;180&deg;, 180&deg;).
      *
      * If point 1 is a pole, the cosine of its latitude is taken to be
@@ -179,8 +180,7 @@ namespace NETGeographicLib {
      * @param[out] lat2 latitude of point 2 (degrees).
      * @param[out] lon2 longitude of point 2 (degrees).
      *
-     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;]; \e lon1 and \e
-     * azi1 should be in the range [&minus;540&deg;, 540&deg;).  The values of
+     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;].  The values of
      * \e lon2 and \e azi2 returned are in the range [&minus;180&deg;,
      * 180&deg;).
      *
@@ -215,14 +215,11 @@ namespace NETGeographicLib {
      * - \e outmask |= Rhumb.LONGITUDE for the latitude \e lon2;
      * - \e outmask |= Rhumb.AREA for the area \e S12;
      * - \e outmask |= Rhumb:.ALL for all of the above;
-     * - \e outmask |= Rhumb.LONG_NOWRAP stops the returned value of \e
-     *   lon2 being wrapped into the range [&minus;180&deg;, 180&deg;).
+     * - \e outmask |= Rhumb.LONG_UNROLL to unroll \e lon2 instead of
+     *   wrapping it into the range [&minus;180&deg;, 180&deg;).
      * .
-     * With the LONG_NOWRAP bit set, the quantity \e lon2 &minus; \e lon1
+     * With the LONG_UNROLL bit set, the quantity \e lon2 &minus; \e lon1
      * indicates how many times the rhumb line wrapped around the ellipsoid.
-     * Because \e lon2 might be outside the normal allowed range for
-     * longitudes, [&minus;540&deg;, 540&deg;), be sure to normalize it with
-     * Math::AngNormalize2 before using it in other GeographicLib calls.
      **********************************************************************/
     void GenDirect(double lat1, double lon1, double azi12, double s12,
                    Rhumb::mask outmask,
@@ -244,8 +241,7 @@ namespace NETGeographicLib {
      * The shortest rhumb line is found.  If the end points are on opposite
      * meridians, there are two shortest rhumb lines and the east-going one is
      * chosen.  \e lat1 and \e lat2 should be in the range [&minus;90&deg;,
-     * 90&deg;]; \e lon1 and \e lon2 should be in the range [&minus;540&deg;,
-     * 540&deg;).  The value of \e azi12 returned is in the range
+     * 90&deg;].  The value of \e azi12 returned is in the range
      * [&minus;180&deg;, 180&deg;).
      *
      * If either point is a pole, the cosine of its latitude is taken to be
@@ -269,8 +265,7 @@ namespace NETGeographicLib {
      * @param[out] azi12 azimuth of the rhumb line (degrees).
      *
      * The shortest rhumb line is found.  \e lat1 and \e lat2 should be in the
-     * range [&minus;90&deg;, 90&deg;]; \e lon1 and \e lon2 should be in the
-     * range [&minus;540&deg;, 540&deg;).  The value of \e azi12 returned is in
+     * range [&minus;90&deg;, 90&deg;].  The value of \e azi12 returned is in
      * the range [&minus;180&deg;, 180&deg;).
      *
      * If either point is a pole, the cosine of its latitude is taken to be
@@ -316,8 +311,7 @@ namespace NETGeographicLib {
      * @param[in] azi12 azimuth of the rhumb line (degrees).
      * @return a RhumbLine object.
      *
-     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;]; \e lon1 and \e
-     * azi12 should be in the range [&minus;540&deg;, 540&deg;).
+     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;].
      *
      * If point 1 is a pole, the cosine of its latitude is taken to be
      * 1/&epsilon;<sup>2</sup> (where &epsilon; is 2<sup>-52</sup>).  This
@@ -420,12 +414,14 @@ namespace NETGeographicLib {
        **********************************************************************/
       AREA          = 1U<<14, //Rhumb::AREA,
       /**
-       * Do wrap the \e lon2 in the direct calculation.
+       * Unroll \e lon2 in the direct calculation.  (This flag used to be
+       * called LONG_NOWRAP.)
        * @hideinitializer
        **********************************************************************/
-      LONG_NOWRAP   = 1U<<14, //Rhumb::LONG_NOWRAP,
+      LONG_UNROLL   = 1U<<15, //Rhumb::LONG_UNROLL,
+      LONG_NOWRAP   = LONG_UNROLL,
       /**
-       * Calculate everything.  (LONG_NOWRAP is not included in this mask.)
+       * Calculate everything.  (LONG_UNROLL is not included in this mask.)
        * @hideinitializer
        **********************************************************************/
       ALL           = 0x7F80U, //Rhumb::ALL,
@@ -490,25 +486,23 @@ namespace NETGeographicLib {
      *
      * @param[in] s12 distance between point 1 and point 2 (meters); it can be
      *   negative.
-     * @param[in] outmask a bitor'ed combination of Rhumb::mask values
+     * @param[in] outmask a bitor'ed combination of RhumbLine::mask values
      *   specifying which of the following parameters should be set.
      * @param[out] lat2 latitude of point 2 (degrees).
      * @param[out] lon2 longitude of point 2 (degrees).
      * @param[out] S12 area under the rhumb line (meters<sup>2</sup>).
      *
-     * The Rhumb::mask values possible for \e outmask are
-     * - \e outmask |= Rhumb::LATITUDE for the latitude \e lat2;
-     * - \e outmask |= Rhumb::LONGITUDE for the latitude \e lon2;
-     * - \e outmask |= Rhumb::AREA for the area \e S12;
-     * - \e outmask |= Rhumb::ALL for all of the above;
-     * - \e outmask |= Rhumb::LONG_NOWRAP stops the returned value of \e
-     *   lon2 being wrapped into the range [&minus;180&deg;, 180&deg;).
+     * The RhumbLine::mask values possible for \e outmask are
+     * - \e outmask |= RhumbLine::LATITUDE for the latitude \e lat2;
+     * - \e outmask |= RhumbLine::LONGITUDE for the latitude \e lon2;
+     * - \e outmask |= RhumbLine::AREA for the area \e S12;
+     * - \e outmask |= RhumbLine::ALL for all of the above;
+     * - \e outmask |= RhumbLine::LONG_UNROLL to unroll \e lon2 instead of
+     *   wrapping it into the range [&minus;180&deg;, 180&deg;).
      * .
-     * With the LONG_NOWRAP bit set, the quantity \e lon2 &minus; \e lon1
-     * indicates how many times the rhumb line wrapped around the ellipsoid.
-     * Because \e lon2 might be outside the normal allowed range for
-     * longitudes, [&minus;540&deg;, 540&deg;), be sure to normalize it with
-     * Math::AngNormalize2 before using it in other GeographicLib calls.
+     * With the LONG_UNROLL bit set, the quantity \e lon2 &minus; \e lon1
+     * indicates how many times and in what sense the rhumb line encircles the
+     * ellipsoid.
      *
      * If \e s12 is large enough that the rhumb line crosses a pole, the
      * longitude of point 2 is indeterminate (a NaN is returned for \e lon2 and
