@@ -97,7 +97,7 @@ namespace GeographicLib {
       CAP_ALL  = 0x1FU,
       CAP_MASK = CAP_ALL,
       OUT_ALL  = 0x7F80U,
-      OUT_MASK = 0xFF80U,       // Includes LONG_NOWRAP
+      OUT_MASK = 0xFF80U,       // Includes LONG_UNROLL
     };
 
     static real CosSeries(real sinx, real cosx, const real c[], int n);
@@ -110,9 +110,9 @@ namespace GeographicLib {
                  real sig12,
                  real ssig1, real csig1, real dn1,
                  real ssig2, real csig2, real dn2,
-                 real cbet1, real cbet2,
+                 real cbet1, real cbet2, unsigned outmask,
                  real& s12s, real& m12a, real& m0,
-                 bool scalep, real& M12, real& M21) const;
+                 real& M12, real& M21) const;
     real InverseStart(EllipticFunction& E,
                       real sbet1, real cbet1, real dn1,
                       real sbet2, real cbet2, real dn2,
@@ -139,7 +139,6 @@ namespace GeographicLib {
       using std::ldexp;
       return ldexp(real(hi), 52) + lo;
     }
-    static const Math::real* rawC4coeff();
 
   public:
 
@@ -203,12 +202,16 @@ namespace GeographicLib {
        **********************************************************************/
       AREA          = 1U<<14 | CAP_C4,
       /**
-       * Do not wrap the \e lon2 in the direct calculation.
+       * Unroll \e lon2 in the direct calculation.  (This flag used to be
+       * called LONG_NOWRAP.)
        * @hideinitializer
        **********************************************************************/
-      LONG_NOWRAP   = 1U<<15,
+      LONG_UNROLL   = 1U<<15,
+      /// \cond SKIP
+      LONG_NOWRAP   = LONG_UNROLL,
+      /// \endcond
       /**
-       * All capabilities, calculate everything.  (LONG_NOWRAP is not
+       * All capabilities, calculate everything.  (LONG_UNROLL is not
        * included in this mask.)
        * @hideinitializer
        **********************************************************************/
@@ -254,8 +257,7 @@ namespace GeographicLib {
      * @param[out] S12 area under the geodesic (meters<sup>2</sup>).
      * @return \e a12 arc length of between point 1 and point 2 (degrees).
      *
-     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;]; \e lon1 and \e
-     * azi1 should be in the range [&minus;540&deg;, 540&deg;).  The values of
+     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;].  The values of
      * \e lon2 and \e azi2 returned are in the range [&minus;180&deg;,
      * 180&deg;).
      *
@@ -368,8 +370,7 @@ namespace GeographicLib {
      *   (dimensionless).
      * @param[out] S12 area under the geodesic (meters<sup>2</sup>).
      *
-     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;]; \e lon1 and \e
-     * azi1 should be in the range [&minus;540&deg;, 540&deg;).  The values of
+     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;].  The values of
      * \e lon2 and \e azi2 returned are in the range [&minus;180&deg;,
      * 180&deg;).
      *
@@ -508,8 +509,8 @@ namespace GeographicLib {
      *   M12 and \e M21;
      * - \e outmask |= GeodesicExact::AREA for the area \e S12;
      * - \e outmask |= GeodesicExact::ALL for all of the above;
-     * - \e outmask |= GeodesicExact::LONG_NOWRAP stops the returned value of
-     *   \e lon2 being wrapped into the range [&minus;180&deg;, 180&deg;).
+     * - \e outmask |= GeodesicExact::LONG_UNROLL to unroll \e lon2 instead of
+     *   wrapping it into the range [&minus;180&deg;, 180&deg;).
      * .
      * The function value \e a12 is always computed and returned and this
      * equals \e s12_a12 is \e arcmode is true.  If \e outmask includes
@@ -517,11 +518,9 @@ namespace GeographicLib {
      * s12_a12.  It is not necessary to include GeodesicExact::DISTANCE_IN in
      * \e outmask; this is automatically included is \e arcmode is false.
      *
-     * With the LONG_NOWRAP bit set, the quantity \e lon2 &minus; \e lon1
-     * indicates how many times the geodesic wrapped around the ellipsoid.
-     * Because \e lon2 might be outside the normal allowed range for
-     * longitudes, [&minus;540&deg;, 540&deg;), be sure to normalize it with
-     * Math::AngNormalize2 before using it in other GeographicLib calls.
+     * With the GeodesicExact::LONG_UNROLL bit set, the quantity \e lon2
+     * &minus; \e lon1 indicates how many times and in what sense the geodesic
+     * encircles the ellipsoid.
      **********************************************************************/
     Math::real GenDirect(real lat1, real lon1, real azi1,
                          bool arcmode, real s12_a12, unsigned outmask,
@@ -551,8 +550,7 @@ namespace GeographicLib {
      * @param[out] S12 area under the geodesic (meters<sup>2</sup>).
      * @return \e a12 arc length of between point 1 and point 2 (degrees).
      *
-     * \e lat1 and \e lat2 should be in the range [&minus;90&deg;, 90&deg;]; \e
-     * lon1 and \e lon2 should be in the range [&minus;540&deg;, 540&deg;).
+     * \e lat1 and \e lat2 should be in the range [&minus;90&deg;, 90&deg;].
      * The values of \e azi1 and \e azi2 returned are in the range
      * [&minus;180&deg;, 180&deg;).
      *
@@ -704,8 +702,7 @@ namespace GeographicLib {
      *   GeodesicLineExact::Position.
      * @return a GeodesicLineExact object.
      *
-     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;]; \e lon1 and \e
-     * azi1 should be in the range [&minus;540&deg;, 540&deg;).
+     * \e lat1 should be in the range [&minus;90&deg;, 90&deg;].
      *
      * The GeodesicExact::mask values are
      * - \e caps |= GeodesicExact::LATITUDE for the latitude \e lat2; this is
