@@ -2,7 +2,7 @@
  * \file Utility.hpp
  * \brief Header for GeographicLib::Utility class
  *
- * Copyright (c) Charles Karney (2011-2014) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2011-2015) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
@@ -299,14 +299,16 @@ namespace GeographicLib {
 #if GEOGRAPHICLIB_PRECISION == 4
       // boost-quadmath treats precision == 0 as "use as many digits as
       // necessary", so...
-      using std::floor;
+      using std::floor; using std::fmod;
       if (p == 0) {
-        long long ix = (long long)(floor(x + Math::real(0.5)));
+        x += Math::real(0.5);
+        Math::real ix = floor(x);
         // Implement the "round ties to even" rule
-        if (Math::real(ix) == x + Math::real(0.5) && (ix % 2) == 1)
-          --ix;
-        s << ix;
-        return s.str();
+        x = (ix == x && fmod(ix, Math::real(2)) == 1) ? ix - 1 : ix;
+        s << std::fixed << std::setprecision(1) << x;
+        std::string r(s.str());
+        // strip off trailing ".0"
+        return r.substr(0, (std::max)(int(r.size()) - 2, 0));
       }
 #endif
       if (p >= 0) s << std::fixed << std::setprecision(p);
@@ -572,7 +574,13 @@ namespace GeographicLib {
      *   256 (i.e., about 77 decimal digits).
      * @return the resulting number of bits of precision.
      *
-     * This only has an effect when GEOGRAPHICLIB_PRECISION == 5.
+     * This only has an effect when GEOGRAPHICLIB_PRECISION == 5.  The
+     * precision should only be set once and before calls to any other
+     * GeographicLib functions.  (Several functions, for example Math::pi(),
+     * cache the return value in a static local variable.  The precision needs
+     * to be set before a call to any such functions.)  In multi-threaded
+     * applications, it is necessary also to set the precision in each thread
+     * (see the example GeoidToGTX.cpp).
      **********************************************************************/
     static int set_digits(int ndigits = 0);
 
