@@ -2,7 +2,7 @@
  * \file Gnomonic.cpp
  * \brief Implementation for GeographicLib::Gnomonic class
  *
- * Copyright (c) Charles Karney (2010-2011) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2010-2015) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
@@ -39,9 +39,8 @@ namespace GeographicLib {
       x = y = Math::NaN();
     else {
       real rho = m/M;
-      azi0 *= Math::degree();
-      x = rho * sin(azi0);
-      y = rho * cos(azi0);
+      Math::sincosd(azi0, x, y);
+      x *= rho; y *= rho;
     }
   }
 
@@ -49,7 +48,7 @@ namespace GeographicLib {
                          real& lat, real& lon, real& azi, real& rk)
     const {
     real
-      azi0 = atan2(x, y) / Math::degree(),
+      azi0 = Math::atan2d(x, y),
       rho = Math::hypot(x, y),
       s = _a * atan(rho/_a);
     bool little = rho <= _a;
