@@ -32,10 +32,8 @@ namespace GeographicLib {
       throw GeographicErr("Scale is not positive");
     if (!(abs(stdlat) <= 90))
       throw GeographicErr("Standard latitude not in [-90d, 90d]");
-    real
-      phi = stdlat * Math::degree(),
-      sphi = sin(phi),
-      cphi = abs(stdlat) != 90 ? cos(phi) : 0;
+    real sphi, cphi;
+    Math::sincosd(stdlat, sphi, cphi);
     Init(sphi, cphi, sphi, cphi, k0);
   }
 
@@ -61,11 +59,10 @@ namespace GeographicLib {
       throw GeographicErr("Standard latitude 1 not in [-90d, 90d]");
     if (!(abs(stdlat2) <= 90))
       throw GeographicErr("Standard latitude 2 not in [-90d, 90d]");
-    real
-      phi1 = stdlat1 * Math::degree(),
-      phi2 = stdlat2 * Math::degree();
-    Init(sin(phi1), abs(stdlat1) != 90 ? cos(phi1) : 0,
-         sin(phi2), abs(stdlat2) != 90 ? cos(phi2) : 0, k1);
+    real sphi1, cphi1, sphi2, cphi2;
+    Math::sincosd(stdlat1, sphi1, cphi1);
+    Math::sincosd(stdlat2, sphi2, cphi2);
+    Init(sphi1, cphi1, sphi2, cphi2, k1);
   }
 
   LambertConformalConic::LambertConformalConic(real a, real f,
@@ -334,7 +331,7 @@ namespace GeographicLib {
   void LambertConformalConic::Forward(real lon0, real lat, real lon,
                                       real& x, real& y, real& gamma, real& k)
     const {
-    lon = Math::AngDiff(Math::AngNormalize(lon0), Math::AngNormalize(lon));
+    lon = Math::AngDiff(lon0, lon);
     // From Snyder, we have
     //
     // theta = n * lambda
@@ -345,10 +342,11 @@ namespace GeographicLib {
     //
     // where nrho0 = n * rho0, drho = rho - rho0
     // and drho is evaluated with divided differences
+    real sphi, cphi;
+    Math::sincosd(Math::LatFix(lat) * _sign, sphi, cphi);
+    cphi = max(epsx_, cphi);
     real
       lam = lon * Math::degree(),
-      phi = _sign * lat * Math::degree(),
-      sphi = sin(phi), cphi = abs(lat) != 90 ? cos(phi) : epsx_,
       tphi = sphi/cphi, scbet = hyp(_fm * tphi),
       scphi = 1/cphi, shxi = sinh(Math::eatanhe(sphi, _es)),
       tchi = hyp(shxi) * tphi - shxi * scphi, scchi = hyp(tchi),
@@ -430,10 +428,9 @@ namespace GeographicLib {
     gamma = atan2(nx, y1);
     real
       tphi = Math::tauf(tchi, _es),
-      phi = _sign * atan(tphi),
       scbet = hyp(_fm * tphi), scchi = hyp(tchi),
       lam = _n ? gamma / _n : x / y1;
-    lat = phi / Math::degree();
+    lat = Math::atand(_sign * tphi);
     lon = lam / Math::degree();
     lon = Math::AngNormalize(lon + Math::AngNormalize(lon0));
     k = _k0 * (scbet/_scbet0) /
