@@ -17,7 +17,7 @@ namespace GeographicLib {
     : _a(a)
     , _f(f <= 1 ? f : 1/f)
     , _e2(_f * (2 - _f))
-    , _es((_f < 0 ? -1 : 0) * sqrt(abs(_e2)))
+    , _es((_f < 0 ? -1 : 1) * sqrt(abs(_e2)))
     , _e2m(1 - _e2)
     , _c( (1 - _f) * exp(Math::eatanhe(real(1), _es)) )
     , _k0(k0)
@@ -61,6 +61,7 @@ namespace GeographicLib {
   void PolarStereographic::Forward(bool northp, real lat, real lon,
                                    real& x, real& y, real& gamma, real& k)
     const {
+    lat = Math::LatFix(lat);
     lat *= northp ? 1 : -1;
     real
       tau = Math::tand(lat),
@@ -71,12 +72,10 @@ namespace GeographicLib {
     rho *= 2 * _k0 * _a / _c;
     k = lat != 90 ? (rho / _a) * secphi * sqrt(_e2m + _e2 / Math::sq(secphi)) :
       _k0;
-    lon = Math::AngNormalize(lon);
-    real
-      lam = lon * Math::degree();
-    x = rho * (lon == -180 ? 0 : sin(lam));
-    y = (northp ? -rho : rho) * (abs(lon) == 90 ? 0 : cos(lam));
-    gamma = northp ? lon : -lon;
+    Math::sincosd(lon, x, y);
+    x *= rho;
+    y *= (northp ? -rho : rho);
+    gamma = Math::AngNormalize(northp ? lon : -lon);
   }
 
   void PolarStereographic::Reverse(bool northp, real x, real y,
@@ -84,15 +83,15 @@ namespace GeographicLib {
     const {
     real
       rho = Math::hypot(x, y),
-      t = rho / (2 * _k0 * _a / _c),
+      t = rho ? rho / (2 * _k0 * _a / _c) :
+      Math::sq(numeric_limits<real>::epsilon()),
       taup = (1 / t - t) / 2,
       tau = Math::tauf(taup, _es),
-      phi = atan(tau),
       secphi = Math::hypot(real(1), tau);
     k = rho ? (rho / _a) * secphi * sqrt(_e2m + _e2 / Math::sq(secphi)) : _k0;
-    lat = (northp ? 1 : -1) * (rho ? phi / Math::degree() : 90);
-    lon = 0 - atan2( -x, northp ? -y : y ) / Math::degree();
-    gamma = northp ? lon : -lon;
+    lat = (northp ? 1 : -1) * Math::atand(tau);
+    lon = Math::atan2d(x, northp ? -y : y );
+    gamma = Math::AngNormalize(northp ? lon : -lon);
   }
 
   void PolarStereographic::SetScale(real lat, real k) {
